// STB_Air 0.0.1 2010:10:28 00:00:00$
// Copyright (c) 2010 stbtrader
// Released under the GNU General Public License
// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
    /// STB_Air. Shows air
    /// http://www.mypivots.com/dictionary/definition/407/air-pocket
    /// http://www.mypivots.com/board/topic/6108/1/chart-of-the-day
    /// http://www.mypivots.com/
	/// </summary>
    [Description("The Air concept was introduced by BruceM at mypivots.")]
	public class STB_Air : Indicator
	{
		//#region Variables
        // options
        private bool _dropAirByTime = true;     // drop air by time
        private bool _dropAirByPrice = false;    // drop air by price
        private bool _dropAirByBar = false;    // drop air by bar
        private bool _drawBarsLimit = true;    // draw max bars
        private bool _dynamicAir = false;    // adapt air
        private bool _drawShades = true;    // draw sades on air
        private int airLookback = 9;     // lookback for support
        private int airLife = 40;     // airLife
        private double airMassFromAtr = 2.0;     // open air
        private double airResistanceFromAtr = 4.0;     // close air
        private int airBarsBegin = 0;
        private int airBarsEnd = 0;
        private int airBarsLimit = 10;

        // parameters
        private int periodAtr = 9;

        // graphics
        private Color _airColorHigh = Color.Red;
        private Color _airColorLow = Color.LightGreen;
        private DashStyle _airDashStyle = DashStyle.Dot;
        private int _airLineWidth = 1;

        // variables
        int inAir = 0;              // is price in air
        double airPriceLimitUp = 0;     // air top
        double airPriceLimitDown = 0;    // air base

        STB_Air_struct air;                 // declare air struct
        List<STB_Air_struct> _airList;      // declare air list

        public struct STB_Air_struct
        {
            public int startBar;
            public int airBar;
            public double basePrice;
            public double priceLimit;
            public int sense;        // -1 down , 1 up
            public string tagLimit; // air<bar>UpT, air<bar>DownT
            public string tagBase; // air<bar>UpB, air<bar>DownB
            public string tagAir; // air<bar>   // tbf
        }

		//#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
 			Overlay			= true;
            CalculateOnBarClose = false;

            STB_Air_struct air = new STB_Air_struct();  // create air 
            _airList = new List<STB_Air_struct>();      // cereate airList
        }
		
		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
            if (CurrentBar < periodAtr) return;
            double tickSize = Bars.Instrument.MasterInstrument.TickSize;

            int lookbackCurrent = Math.Min(CurrentBar, airLookback);
            double atr = ATR(BarsArray[0], periodAtr)[0];
            double airMargin = atr / airMassFromAtr;
            double airWeight = atr / airResistanceFromAtr;

            airPriceLimitUp = MAX(High, lookbackCurrent)[1];
            airPriceLimitDown = MIN(Low, lookbackCurrent)[1];
            
            // create air
            if (inAir == 0)
            {
                // can be air up and down at the same time
                if ((Value[0] > (airPriceLimitUp + (atr / airMassFromAtr)))
                    || (Value[0] < airPriceLimitDown - (atr / airMassFromAtr)))
                {
                    air.startBar = CurrentBar;
                    air.airBar = CurrentBar;
                    air.priceLimit = Value[0];
                    air.tagLimit = "air" + CurrentBar + "L";
                    air.tagBase = "air" + CurrentBar + "B";
                    air.tagAir = "air" + CurrentBar;
                    if (Value[0] > airPriceLimitUp)      // air up
                    {
                        air.sense = 1;
                        air.basePrice = airPriceLimitUp;
                    }
                    if (Value[0] < airPriceLimitDown)
                    {
                        air.sense = -1;   // air down
                        air.basePrice = airPriceLimitDown;
                    }
                    inAir = 1;
                }
            }

            // change airBar
            if (inAir == 1)
            {
                air.airBar = CurrentBar;
            }

            // move airLimit
            if (inAir == 1)
            {
                if (air.sense == 1)        // up air
                {
                    if (Value[0] > air.priceLimit)
                    {
                        air.priceLimit = Value[0];
                    }
                    if ((Value[0] < air.priceLimit) && (_dynamicAir))
                    {
                        air.priceLimit = Value[0];
                    }
                }
                else if (air.sense == -1)  // down air
                {
                    if (Value[0] < air.priceLimit)
                    {
                        air.priceLimit = Value[0];
                    }
                    if ((Value[0] > air.priceLimit) && (_dynamicAir))
                    {
                        air.priceLimit = Value[0];
                    }
                }
            }

            // save air
            if (inAir == 1)
            {
                if (air.sense == 1)        // up air
                {
                    if (Value[0] < (High[0] - (atr / airResistanceFromAtr)))       // air closed
                    {
                        _airList.Add(air);          // add air up
                        airPriceLimitUp = High[0];  // set new airTop
                        inAir = 0;                  // no current air
                    }
                }
                else if (air.sense == -1)
                {
                    if (Value[0] > (Low[0] + (atr / airResistanceFromAtr)))       // air closed
                    {
                        _airList.Add(air);          // add air down
                        airPriceLimitDown = Low[0]; // set new airTop
                        inAir = 0;                  // no current air
                    }
                }
            }

            // drop airs by Price
            if (_dropAirByPrice)
            {
                if (_airList != null)
                {
                    _airList.ForEach(airItem =>
                    {
                        if (airItem.sense == 1)       // up air
                        {
                            if (Low[0] <= airItem.basePrice)
                            {
                                _airList.Remove(airItem);
                                if (_drawShades)
                                {
                                    RemoveDrawObject(airItem.tagAir);    // region // tbf
                                }
                                else
                                {
                                    RemoveDrawObject(airItem.tagLimit);    // top line
                                    RemoveDrawObject(airItem.tagBase);    // base line
                                }
                            }
                        }
                    });
                }

                if (_airList != null)
                {
                    _airList.ForEach(airItem =>
                    {
                        if (airItem.sense == -1)       // down air
                        {
                            if (High[0] >= airItem.priceLimit)
                            {
                                _airList.Remove(airItem);
                                if (_drawShades)
                                {
                                    RemoveDrawObject(airItem.tagAir);    // region // tbf
                                }
                                else
                                {
                                    RemoveDrawObject(airItem.tagLimit);    // top line
                                    RemoveDrawObject(airItem.tagBase);    // base line
                                }
                            }
                        }
                    });
                }
            }

            // drop airs by time
            if (_dropAirByTime)
            {
                if (_airList != null)
                {
                    _airList.ForEach(airItem =>
                    {
                        int airLastBar = Bars.Count - airLife;

                        if (airItem.airBar < airLastBar)
                        {
                            _airList.Remove(airItem);
                            if (_drawShades)
                            {
                                RemoveDrawObject(airItem.tagAir);    // region // tbf
                            }
                            else
                            {
                                RemoveDrawObject(airItem.tagLimit);    // top line
                                RemoveDrawObject(airItem.tagBase);    // base line
                            }
                        }
                    });
                }
            }
 
            // drop airs by bar
            if (_dropAirByBar)
            {
                if (_airList != null)
                {
                    _airList.ForEach(airItem =>
                    {
                        int airLastBar = Bars.Count - airLife;

                        if ((airItem.airBar < airBarsBegin) || (airItem.airBar > airBarsEnd))
                        {
                            _airList.Remove(airItem);
                            if (_drawShades)
                            {
                                RemoveDrawObject(airItem.tagAir);    // region // tbf
                            }
                            else
                            {
                                RemoveDrawObject(airItem.tagLimit);    // top line
                                RemoveDrawObject(airItem.tagBase);    // base line
                            }
                        }
                    });
                }
            }

            // draw airs
            if (inAir == 1)
            {
                int barsBack = 0;
                if (_drawBarsLimit)
                {
                    barsBack = Math.Max(0, CurrentBar - (air.startBar + airBarsLimit));
                }

                if (_drawShades)
                {
                    if (air.sense == 1)
                        DrawRectangle("air" + air.airBar, false, CurrentBar - air.startBar, air.basePrice, barsBack, air.priceLimit, Color.Empty, _airColorHigh, 4);
                    else if (air.sense == -1)
                        DrawRectangle("air" + air.airBar, false, CurrentBar - air.startBar, air.basePrice, barsBack, air.priceLimit, Color.Empty, _airColorLow, 4);
                }
                else
                {
                    DrawLine(air.tagLimit, false, CurrentBar - air.startBar, air.priceLimit, barsBack, air.priceLimit, _airColorHigh, _airDashStyle, _airLineWidth);
                    DrawLine(air.tagBase, false, CurrentBar - air.startBar, air.basePrice, barsBack, air.basePrice, _airColorLow, _airDashStyle, _airLineWidth);
                }
            }
  
            if (_airList != null)
            {
                _airList.ForEach(airItem =>
                {
                    int barsBack = 0;
                    if (_drawBarsLimit)
                    {
                        barsBack = Math.Max(0, CurrentBar - (airItem.startBar + airBarsLimit));
                    }

                    if (_drawShades)
                    {
                        if (airItem.sense == 1)
                            DrawRectangle(airItem.tagAir, false, CurrentBar - airItem.startBar, airItem.basePrice, barsBack, airItem.priceLimit, Color.Empty, _airColorHigh, 4);
                        else if (airItem.sense == -1)
                            DrawRectangle(airItem.tagAir, false, CurrentBar - airItem.startBar, airItem.basePrice, barsBack, airItem.priceLimit, Color.Empty, _airColorLow, 4);
                    }
                    else
                    {
                        DrawLine(airItem.tagLimit, false, CurrentBar - airItem.startBar, airItem.priceLimit, barsBack, airItem.priceLimit, _airColorHigh, _airDashStyle, _airLineWidth);
                        DrawLine(airItem.tagBase, false, CurrentBar - airItem.startBar, airItem.basePrice, barsBack, airItem.basePrice, _airColorLow, _airDashStyle, _airLineWidth);
                    }
                });
            }
        }
		#region Properties

        /// <summary>
        /// </summary>
        [Description("ATR period to evaluate air")]
        [GridCategory("Parameters")]
        public int PeriodAtr
        {
            get { return periodAtr; }
            set { periodAtr = Math.Max(1, value); }
        }
        /// <summary>
        /// </summary>
        [Description("Remove air by time")]
        [GridCategory("Options")]
        public bool DropAirByTime
        {
            get { return _dropAirByTime; }
            set { _dropAirByTime = value; }
        }
        /// <summary>
        /// </summary>
        [Description("Remove air by price")]
        [GridCategory("Options")]
        public bool DropAirByPrice
        {
            get { return _dropAirByPrice; }
            set { _dropAirByPrice = value; }
        }
        /// <summary>
        /// </summary>
        [Description("Remove air by bar")]
        [GridCategory("Options")]
        public bool DropAirByBar
        {
            get { return _dropAirByBar; }
            set { _dropAirByBar = value; }
        }
        /// <summary>
        /// </summary>
        [Description("Adapt air")]
        [GridCategory("Options")]
        public bool DynamicAir
        {
            get { return _dynamicAir; }
            set { _dynamicAir = value; }
        }
        /// <summary>
        /// </summary>
        [Description("Draw shades")]
        [GridCategory("Options")]
        public bool DrawShades
        {
            get { return _drawShades; }
            set { _drawShades = value; }
        }

        /// <summary>
        /// </summary>
        [Description("Look back to find air")]
        [GridCategory("Options")]
        public int AirLookback
        {
            get { return airLookback; }
            set { airLookback = Math.Max(1, value); }
        }
        /// <summary>
        /// </summary>
        [Description("Air Life")]
        [GridCategory("Options")]
        public int AirLife
        {
            get { return airLife; }
            set { airLife = Math.Max(1, value); }
        }

        /// <summary>
        /// </summary>
        [Description("Air Mass")]
        [GridCategory("Options")]
        public double AirMassFromAtr
        {
            get { return airMassFromAtr; }
            set { airMassFromAtr = Math.Max(1, value); }
        }
        /// <summary>
        /// </summary>
        [Description("Air Resistance")]
        [GridCategory("Options")]
        public double AirResistanceFromAtr
        {
            get { return airResistanceFromAtr; }
            set { airResistanceFromAtr = Math.Max(1, value); }
        }

        /// <summary>
        /// </summary>
        [Description("Air Bars Begin")]
        [GridCategory("Options")]
        public int AirBarsBegin
        {
            get { return airBarsBegin; }
            set { airBarsBegin = Math.Max(0, value); }
        }
        /// <summary>
        /// </summary>
        [Description("Air Bars End")]
        [GridCategory("Options")]
        public int AirBarsEnd
        {
            get { return airBarsEnd; }
            set { airBarsEnd = Math.Max(0, value); }
        }
        /// <summary>
        /// </summary>
        [Description("Draw max bars")]
        [GridCategory("Options")]
        public bool DrawBarsLimit
        {
            get { return _drawBarsLimit; }
            set { _drawBarsLimit = value; }
        }
        /// <summary>
        /// </summary>
        [Description("Max bars to draw")]
        [GridCategory("Options")]
        public int AirBarsLimit
        {
            get { return airBarsLimit; }
            set { airBarsLimit = Math.Max(1, value); }
        }

        // Colors
        [XmlIgnore()]
        [Description("AirHigh_Color")]
        [Category("Colors")]
        public Color STB_AirHigh_Color
        {
            get { return _airColorHigh; }
            set { _airColorHigh = value; }
        }
        [Browsable(false)]
        public string STB_AirHigh_ColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(STB_AirHigh_Color); }
            set { STB_AirHigh_Color = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [XmlIgnore()]
        [Description("AirLow_Color")]
        [Category("Colors")]
        public Color STB_AirLow_Color
        {
            get { return _airColorLow; }
            set { _airColorLow = value; }
        }
        [Browsable(false)]
        public string STB_AirLow_ColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(STB_AirLow_Color); }
            set { STB_AirLow_Color = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private STB_Air[] cacheSTB_Air = null;

        private static STB_Air checkSTB_Air = new STB_Air();

        /// <summary>
        /// The Donchian Channel indicator was created by Richard Donchian. It uses the highest high and the lowest low of a period of time to plot the channel.
        /// </summary>
        /// <returns></returns>
        public STB_Air STB_Air(int periodAtr)
        {
            return STB_Air(Input, periodAtr);
        }

        /// <summary>
        /// The STB_Air indicator shows lines on triple high or low values as described by BruceM.
        /// </summary>
        /// <returns></returns>
        public STB_Air STB_Air(Data.IDataSeries input, int periodAtr)
        {
            if (cacheSTB_Air != null)
                for (int idx = 0; idx < cacheSTB_Air.Length; idx++)
                    if (cacheSTB_Air[idx].PeriodAtr == periodAtr && cacheSTB_Air[idx].EqualsInput(input))
                        return cacheSTB_Air[idx];

            lock (checkSTB_Air)
            {
                checkSTB_Air.PeriodAtr = periodAtr;
                periodAtr = checkSTB_Air.PeriodAtr;

                if (cacheSTB_Air != null)
                    for (int idx = 0; idx < cacheSTB_Air.Length; idx++)
                        if (cacheSTB_Air[idx].PeriodAtr == periodAtr && cacheSTB_Air[idx].EqualsInput(input))
                            return cacheSTB_Air[idx];

                STB_Air indicator = new STB_Air();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.PeriodAtr = periodAtr;
                Indicators.Add(indicator);
                indicator.SetUp();

                STB_Air[] tmp = new STB_Air[cacheSTB_Air == null ? 1 : cacheSTB_Air.Length + 1];
                if (cacheSTB_Air != null)
                    cacheSTB_Air.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSTB_Air = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The STB_Air indicator shows lines on triple high or low values as described by BruceM.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.STB_Air STB_Air(int periodAtr)
        {
            return _indicator.STB_Air(Input, periodAtr);
        }

        /// <summary>
        /// The STB_Air indicator shows lines on triple high or low values as described by BruceM.
        /// </summary>
        /// <returns></returns>
        public Indicator.STB_Air STB_Air(Data.IDataSeries input, int periodAtr)
        {
            return _indicator.STB_Air(input, periodAtr);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The STB_Air indicator shows lines on triple high or low values as described by BruceM.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.STB_Air STB_Air(int periodAtr)
        {
            return _indicator.STB_Air(Input, periodAtr);
        }

        /// <summary>
        /// The STB_Air indicator shows lines on triple high or low values as described by BruceM.
        /// </summary>
        /// <returns></returns>
        public Indicator.STB_Air STB_Air(Data.IDataSeries input, int periodAtr)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.STB_Air(input, periodAtr);
        }
    }
}
#endregion
